#!/usr/bin/env python -------------------------------- -*- coding: utf-8 -*-#
#                      2023 3DMish <Mish7913@gmail.com>                     #

# -----              ##### BEGIN GPL LICENSE BLOCK #####              ----- #
#                                                                           #
#  This  program  is  free  software;   you  can  redistribute  it  and/or  #
#  modify  it  under  the  terms  of   the   GNU  General  Public  License  #
#  as  published  by  the  Free  Software  Foundation;  either  version  2  #
#  of the License, or (at your option) any later version.                   #
#                                                                           #
#  This program  is  distributed  in the hope  that  it  will  be  useful,  #
#  but  WITHOUT  ANY  WARRANTY;  without  even  the  implied  warranty  of  #
#  MERCHANTABILITY  or  FITNESS   FOR  A  PARTICULAR  PURPOSE.    See  the  #
#  GNU General Public License for more details.                             #
#                                                                           #
#  You  should  have  received  a  copy  of the GNU General Public License  #
#  along with this program; if not, write to the Free Software Foundation,  #
#  Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.       #
#                                                                           #
# -----               ##### END GPL LICENSE BLOCK #####               ----- #

import bpy, os, sys;

sys.path.insert(0, os.path.dirname(os.path.abspath(__file__)) + "/");
sys.path.insert(0, os.path.dirname(os.path.abspath(__file__)) + "/../");

from bpy.types import Header, Panel, Operator;

from bpy_sys import (
    q_register_class, q_unregister_class, get_prop, lc_cont_x,
    r_register_class, r_unregister_class, lc_icon,
);

import m7a_mesh_tool,   m7a_gpencil_tool, m7a_armature_tool;
import m7a_other_tools, m7a_curve_tool;

bl_conf = {
    "VIEW3D_HT_tool_header": None,
}

class VIEW3D_HT_tool_header (Header):
    bl_space_type  = 'VIEW_3D';
    bl_region_type = 'TOOL_HEADER';
    
    def draw(self, context):
        lc_main = self.layout.row(align = True);
        
        lc_tool = context.workspace.tools.from_space_view3d_mode(context.mode);
        
        lc_main.operator("ed.undo", icon="LOOP_BACK", text="");
        lc_main.operator("ed.redo", icon="LOOP_FORWARDS", text="");
        
        if (lc_tool.idname == 'builtin.spin'):
            lc_main.separator();
            
            lc_row = lc_main.row(align = True);
            lc_row.ui_units_x = 8;
            lc_row.prop(lc_tool.operator_properties("mesh.spin"), "steps", text="");
            lc_row.prop(lc_tool.gizmo_group_properties("MESH_GGT_spin"), "axis");
        
        lc_main.separator();
        
        a_obj = context.active_object;
        if (a_obj != None):
            if   (a_obj.type == "MESH"):     m7a_mesh_tool.draw_panel(lc_main, context);
            elif (a_obj.type == "CURVE"):    m7a_curve_tool.draw_panel(lc_main, context);
            elif (a_obj.type == "ARMATURE"): m7a_armature_tool.draw_panel(lc_main, context);
            elif (a_obj.type == "GPENCIL"):  m7a_gpencil_tool.draw_panel(lc_main, context);
            else:                            m7a_other_tools.draw_panel(lc_main, context);
        
        lc_main.separator_spacer();
        
        def lc_mirror(icon = 0):
            if (icon == 0): lc_main.label(icon='MOD_MIRROR');
            lc_row = lc_main.row(align = True);
            lc_row.ui_units_x = 0.8;
            return lc_row;
            
        if (context.mode == 'EDIT_ARMATURE'):
            lc_mirror().prop(context.object.data, "use_mirror_x", text="X", toggle=True);
            lc_main.separator();
        elif (context.mode == 'POSE'):
            lc_mirror().prop(context.object.pose, "use_mirror_x", text="X", toggle=True);
            lc_main.separator();
        elif (context.mode in {'EDIT_MESH', 'PAINT_WEIGHT', 'SCULPT', 'PAINT_VERTEX', 'PAINT_TEXTURE'}):
            lc_mirror(0).prop(context.object, "use_mesh_mirror_x", text="X", toggle=True);
            lc_mirror(1).prop(context.object, "use_mesh_mirror_y", text="Y", toggle=True);
            lc_mirror(2).prop(context.object, "use_mesh_mirror_z", text="Z", toggle=True);
            if (context.mode == 'SCULPT'):
                lc_main.popover(panel="VIEW3D_PT_sculpt_symmetry_for_topbar", text="");
            if (context.mode == 'PAINT_VERTEX'):
                lc_main.popover(panel="VIEW3D_PT_tools_vertexpaint_symmetry_for_topbar", text="");
            if (context.mode == 'PAINT_WEIGHT'):
                lc_main.popover(panel="VIEW3D_PT_tools_weightpaint_symmetry_for_topbar", text="");
            lc_main.separator();
            
        popover_kw = {"space_type": 'VIEW_3D', "region_type": 'UI', "category": "Tool"};

        if (context.mode == 'SCULPT'):         lc_main.popover_group(context=".sculpt_mode",   **popover_kw);
        elif (context.mode == 'PAINT_VERTEX'): lc_main.popover_group(context=".vertexpaint",   **popover_kw);
        elif (context.mode == 'PAINT_WEIGHT'): lc_main.popover_group(context=".weightpaint",   **popover_kw);
        elif (context.mode == 'PAINT_TEXTURE'):lc_main.popover_group(context=".imagepaint",    **popover_kw);
        elif (context.mode == 'EDIT_TEXT'):    lc_main.popover_group(context=".text_edit",     **popover_kw);
        elif (context.mode == 'EDIT_ARMATURE'):lc_main.popover_group(context=".armature_edit", **popover_kw);
        elif (context.mode == 'EDIT_METABALL'):lc_main.popover_group(context=".mball_edit",    **popover_kw);
        elif (context.mode == 'EDIT_LATTICE'): lc_main.popover_group(context=".lattice_edit",  **popover_kw);
        elif (context.mode == 'EDIT_CURVE'):   lc_main.popover_group(context=".curve_edit",    **popover_kw);
        elif (context.mode == 'EDIT_MESH'):
            lc_main.prop(context.tool_settings, "use_mesh_automerge", text="");
            lc_main.separator();
        elif (context.mode == 'POSE'):     lc_main.popover_group(context=".posemode",     **popover_kw);
        elif (context.mode == 'PARTICLE'): lc_main.popover_group(context=".particlemode", **popover_kw);
        
        if (context.mode in {'OBJECT', 'EDIT_MESH'}):
            lc_main.popover(panel="M7A_PT_OPTIONS_panel", text="Options");
        
        if (context.mode in {"EDIT_GPENCIL"}):
            lc_main.operator("gpencil.snap_cursor_to_selected", text="", icon="PIVOT_CURSOR");
        else:
            lc_main.operator("view3d.snap_cursor_to_selected", text="", icon="PIVOT_CURSOR");
        lc_main.operator("view3d.snap_cursor_to_center",   text="", icon="CURSOR");

def register_nodes (self, context):
    if get_prop("view3d_upgrade_header") and \
        get_prop("view3d_tool_header"):
            register();
    else: unregister();

class M7A_PT_OPTIONS_panel (Panel):
    bl_space_type  = 'VIEW_3D'
    bl_region_type = 'HEADER'
    bl_label       = "Options";
    
    @staticmethod
    def draw(self, context):
        lc_main = self.layout.column(align=True);
        lc_tool = context.workspace.tools.from_space_view3d_mode(context.mode).idname;
        
        lc_main.separator();
        
        lc_main.label(text="Tool Options", icon="DOT");
        
        lc_row = lc_main.row(align = True);
        if (lc_tool == 'builtin.select_box'):
            lc_props = context.workspace.tools[lc_tool].operator_properties("view3d.select_box");
            lc_row.prop(lc_props, "mode", text="", expand=True, icon_only=True);
        elif (lc_tool == 'builtin.move'):
            lc_row.prop(context.scene.transform_orientation_slots[1], "type", text="");
            lc_row.popover(panel="TOPBAR_PT_tool_fallback", text="Select");
            
        if (context.mode in {'OBJECT'}):
            lc_main.separator();
        
            lc_main.label(text="Object Options", icon="OBJECT_DATA");
            
            lc_main.separator();
            
            lc_main.prop(context.tool_settings, "use_transform_data_origin",       toggle=True);
            lc_main.prop(context.tool_settings, "use_transform_pivot_point_align", toggle=True);
            lc_main.prop(context.tool_settings, "use_transform_skip_children",     toggle=True);
            
        elif (context.mode in {'EDIT_MESH'}):
            lc_main.separator();
            
            lc_main.label(text="Mesh Options", icon="MESH_DATA");
            
            lc_main.separator();
            
            lc_main.prop(context.tool_settings, "use_transform_correct_face_attributes", toggle=True);
            lc_main.prop(context.tool_settings, "use_transform_correct_keep_connected",  toggle=True);
            
            lc_main.separator();
            
            lc_main.prop(context.tool_settings, "use_edge_path_live_unwrap", text="UV Live Unwrap", toggle=True);
            
            lc_main.separator();
            
            lc_row = lc_main.row(align = True);
            lc_row.label(icon='MOD_MIRROR');
            lc_row_xyz = lc_cont_x(lc_row, 0.5);
            lc_row_xyz.prop(context.object, "use_mesh_mirror_x", text="X", toggle=True);
            lc_row_xyz.prop(context.object, "use_mesh_mirror_y", text="Y", toggle=True);
            lc_row_xyz.prop(context.object, "use_mesh_mirror_z", text="Z", toggle=True);
            lc_row.separator();
            lc_row.prop(context.active_object.data, "use_mirror_topology", text="Topology", toggle=True);
            
            lc_main.separator();
            
            lc_row = lc_main.row(align = True);
            lc_row.prop(context.tool_settings, "use_mesh_automerge", text="Auto Merge", toggle=True);
            lc_active = context.tool_settings.use_mesh_automerge;
            lc_cont_x(lc_row, 0.8, active=lc_active).prop(
                context.tool_settings, "use_mesh_automerge_and_split", text="Split", toggle=True
            );
            lc_cont_x(lc_main, active=lc_active).prop(
                context.tool_settings, "double_threshold", text="Threshold", toggle=True
            );
            
            lc_main.separator();
            
            lc_row = lc_main.row(align = True);
            lc_row.operator("mesh.intersect_boolean", icon="MOD_BOOLEAN", text="Boolean");
            lc_row.operator("mesh.intersect_multi_boolean", icon="MOD_BOOLEAN", text="Multi Union");
            
            lc_row = lc_main.row(align = True);
            lc_row.operator("transform.vertex_random", icon="MOD_NOISE", text="Noise").offset=0.05;
            lc_row.operator("mesh.vertices_smooth", icon="MOD_SMOOTH", text="Smooth").factor=0.5;
            lc_row.operator("mesh.blend_from_shape", text="", icon="SHAPEKEY_DATA");
            
            lc_main.separator();
            
            lc_row = lc_main.row(align = True);
            lc_row.operator("mesh.subdivide", icon="MOD_SUBSURF", text="Subdivide").smoothness=1;
            lc_row.operator("mesh.unsubdivide", text="UN-Subdivide", icon="MOD_DECIM").iterations = 2;
            
            lc_main.separator();
            
            lc_row = lc_main.row(align = True);
            lc_cont_x(lc_row, 0.9).operator(
                "view3d.edit_mesh_extrude_move_shrink_fatten", text="Extrude", 
                icon_value=lc_icon("EXTRUDE_NORMALS")
            );
            lc_btn = lc_row.operator(
                "view3d.edit_mesh_extrude_move_normal", text="Intersect", icon_value=lc_icon("EXTRUDE_INTERSECT")
            ); lc_btn.dissolve_and_intersect = True;
            lc_cont_x(lc_row, 0.8).operator("mesh.inset", text="Inset", icon_value=lc_icon("INSET"));
            
            lc_main.separator();
            
            lc_row = lc_main.row(align = True);
            lc_row.operator("mesh.edge_face_add", text="Add Edge/Face", icon="FACE_MAPS");
            lc_cont_x(lc_row, 0.8).operator("mesh.bridge_edge_loops", text="Bridge", icon_value=lc_icon("BRIDGE"));
            
            lc_row = lc_main.row(align = True);
            lc_row.operator("mesh.fill", text="Fill", icon_value=lc_icon("FILL"));
            lc_row.operator("mesh.fill_grid", text="Fill Grid", icon_value=lc_icon("FILL_GRID"));
            lc_btn = lc_row.operator("mesh.quads_convert_to_tris", text="", icon="MOD_TRIANGULATE");
            lc_btn.quad_method="BEAUTY"; lc_btn.ngon_method="BEAUTY";
            lc_row.operator("mesh.tris_convert_to_quads", text="", icon_value=lc_icon("TO_QUADS"));
        

class OBJECT_OT_MULTI_BOOLEAN_Union (Operator):
    bl_idname      = "mesh.intersect_multi_boolean";
    bl_label       = "Multi Boolean Union";
    bl_options     = {'REGISTER', 'UNDO'};
    bl_description = "Multi Intersect Boolean Union";
    
    def execute(self, context):
        s_obj = context.selected_objects;
        a_obj = context.active_object;
        parts = [];
        
        bpy.ops.mesh.select_linked();
        bpy.ops.object.mode_set(mode='OBJECT', toggle=False);
        
        for obj in s_obj:
            if (obj != a_obj):
                obj.select_set(False);
                
        bpy.ops.object.mode_set(mode='EDIT', toggle=False);
        bpy.ops.mesh.hide(unselected=True);
        bpy.ops.mesh.select_all(action='DESELECT');
        
        bpy.ops.object.mode_set(mode='OBJECT', toggle=False);
        
        while 1:
            stop = True;
            for vert in a_obj.data.vertices:
                if (vert.hide == False):
                    vert.select = True;
                    stop = False;
                    break;
            if (stop == True): break;
        
            bpy.ops.object.mode_set(mode='EDIT', toggle=False);
            
            bpy.ops.mesh.select_linked();
            bpy.ops.mesh.separate(type='SELECTED');
            
            bpy.ops.object.mode_set(mode='OBJECT', toggle=False);
            
            for obj in context.selected_objects:
                if (obj != a_obj):
                    obj.select_set(False);
                    parts.append(obj);
        
        bpy.ops.object.select_all(action='DESELECT');
        
        parts[0].select_set(True);
        bpy.context.view_layer.objects.active = parts[0];
        
        for obj in parts:
            if (obj != parts[0]):
                modifier = parts[0].modifiers.new(name="M7A_Boonean", type='BOOLEAN');
                modifier.operation = 'UNION';
                modifier.solver = 'FAST';
                modifier.object = obj;
                bpy.ops.object.modifier_apply(modifier="M7A_Boonean");
        
        bpy.ops.object.select_all(action='DESELECT');

        for obj in parts:
            if (obj != parts[0]):
                obj.select_set(True);
                bpy.context.view_layer.objects.active = obj;
                bpy.ops.object.delete(use_global=False, confirm=False);

        parts[0].select_set(True);
        a_obj.select_set(True);
        bpy.context.view_layer.objects.active = a_obj;
        bpy.ops.object.join();
        
        bpy.ops.object.mode_set(mode='EDIT', toggle=False);
        bpy.ops.mesh.select_all(action='DESELECT');
        bpy.ops.mesh.reveal();
        bpy.ops.mesh.select_all(action='INVERT');
        
        return {'FINISHED'};

classes = [
    VIEW3D_HT_tool_header, M7A_PT_OPTIONS_panel, OBJECT_OT_MULTI_BOOLEAN_Union,
];

def register():
    r_unregister_class(bl_conf, "VIEW3D_HT_tool_header");
    
    m7a_armature_tool.register();
    m7a_gpencil_tool.register();
    m7a_other_tools.register();
    m7a_mesh_tool.register();
    
    q_register_class(classes);
    
def unregister():
    q_unregister_class(classes);
    
    m7a_armature_tool.unregister();
    m7a_gpencil_tool.unregister();
    m7a_other_tools.unregister();
    m7a_mesh_tool.unregister();
    
    r_register_class(bl_conf, "VIEW3D_HT_tool_header");
